package com.example.demo.controller;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Console;
import org.apache.rocketmq.client.consumer.DefaultLitePullConsumer;
import org.apache.rocketmq.client.exception.MQClientException;
import org.apache.rocketmq.client.producer.DefaultMQProducer;
import org.apache.rocketmq.common.consumer.ConsumeFromWhere;
import org.apache.rocketmq.common.message.Message;
import org.apache.rocketmq.common.message.MessageExt;

import java.nio.charset.StandardCharsets;
import java.util.List;
/**
 * @author xrp
 * @date 2021/4/14 9:39
 */

public class Reliability {
//    private static String topicName = "test";   // topic名
//    private static String group = "test-group"; // 消费组名
//    private static String namesrvAddress = "100.85.124.123:8600;100.85.126.201:8600"; // 元数据链接地址

    private static String topicName = "miyapaytest";   // topic名
    private static String group = "miyapaytest_consumer"; // 消费组名
    private static String namesrvAddress = "172.16.11.248:8100;172.16.10.101:8100"; // 元数据链接地址

    public static void main(String[] args) throws Exception {
//        produce();
//        consume();
        for (int i = 0; i < 10; i++) {

            System.out.println(String.format("message-%s-%d", System.currentTimeMillis(),i));
        }
    }

    public static void consume() throws MQClientException {
        DefaultLitePullConsumer consumer = new DefaultLitePullConsumer(group);
        consumer.setAutoCommit(true);
        consumer.setNamesrvAddr(namesrvAddress);
        consumer.setInstanceName("Test" + System.currentTimeMillis());
        consumer.subscribe(topicName, "*");
        consumer.setConsumeFromWhere(ConsumeFromWhere.CONSUME_FROM_FIRST_OFFSET);
        consumer.setAutoCommit(true);
        consumer.start();

        try {
            for (int i = 0; i < 20; i++) {
                List<MessageExt> messages = consumer.poll();
                messages.get(i).getBody().toString();
                System.out.printf("%s%n", messages);
            }
        } finally {
            consumer.shutdown();
        }
    }
    private static boolean runFlag = true;
    public static void PullConsumer() throws MQClientException {
        DefaultLitePullConsumer consumer = new DefaultLitePullConsumer(group);
        consumer.setAutoCommit(true);
        consumer.setNamesrvAddr(namesrvAddress);
        consumer.setInstanceName("Test" + System.currentTimeMillis());
        //要消费的topic，可使用tag进行简单过滤
        consumer.subscribe(topicName, "*");
        consumer.setConsumeFromWhere(ConsumeFromWhere.CONSUME_FROM_FIRST_OFFSET);
        //设置是否自动提交offset，默认值是true
        consumer.setAutoCommit(false);

//        //一次最大消费的条数
//        consumer.setPullBatchSize(100);
//         //无消息时，最大阻塞时间。默认5000 单位ms
//        consumer.setPollTimeoutMillis(5000);

        consumer.start();

        try {
            while (runFlag){
                try {
                    //拉取消息，无消息时会阻塞
                    List<MessageExt> msgs = consumer.poll();
                    if (CollUtil.isEmpty(msgs)){
                        continue;
                    }
                    //业务处理
                    msgs.forEach(msg-> Console.log(new String(msg.getBody())));
                    //同步消费位置。不执行该方法，应用重启会存在重复消费。
  //由于前面调用setAutoCommit方法将自动提交位点属性设置为false，
   // 所以这里调用commitSync将消费位点提交到内存中的offsetstore，最终会通过定时任务将消费位点提交给broker
                    consumer.commitSync();
                    /**
                     DefaultLitePullConsumer的实现提供了以下特性：

                     订阅方式消费消息支持消息队列负载均衡
                     分配方式消费消息支持收到分配消息队列，此时不支持负载均衡
                     提供了seek方法方便用户重置消费位点
                     提供commitSync方法方便用户手动提交消费位点
                     */
                }catch (Exception e){
                    e.printStackTrace();
                }
            }
        } finally {
            consumer.shutdown();
        }
    }
    public static void produce() throws Exception {
        DefaultMQProducer producer = new DefaultMQProducer("Test" + System.currentTimeMillis());
        producer.setNamesrvAddr(namesrvAddress);

        producer.start();

        Message message;

        for (int i = 0; i < 10; i++) {
            message = new Message(topicName, String.format("message-%s-%d", System.currentTimeMillis(), i).getBytes(
                    StandardCharsets.UTF_8));
            System.out.println(producer.send(message));
        }

        producer.shutdown();
    }
}

