/*
 * Miya.com Inc.
 * Copyright (c) 2004-2018 All Rights Reserved.
 */
package com.example.demo.mq;

import com.example.demo.mq.listener.LedgerConsumerListener;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.consumer.DefaultMQPushConsumer;
import org.apache.rocketmq.client.exception.MQClientException;
import org.apache.rocketmq.common.consumer.ConsumeFromWhere;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Component;

@Slf4j
@Component
public class ConsumerConfiguration {

    @Autowired
    private LedgerConsumerListener ledgerConsumerListener;


    /**
     * 每配置一个代表一个consumer，可以配置多个
     *
     * @return
     */
    @Bean(name = "consumerPayNotice", initMethod = "start", destroyMethod = "shutdown")
    DefaultMQPushConsumer consumerPayNotice() throws MQClientException {
        DefaultMQPushConsumer consumer = new DefaultMQPushConsumer("miyapaytest_consumer");
        consumer.setNamesrvAddr("172.16.10.101:8100");
        //消费模式:一个新的订阅组第一次启动从队列的最后位置开始消费 后续再启动接着上次消费的进度开始消费
        consumer.setConsumeFromWhere(ConsumeFromWhere.CONSUME_FROM_LAST_OFFSET);
        //订阅主题和 标签（ * 代表所有标签)下信息
        consumer.subscribe("miyapaytest", "tag");

        //设置多线程消费
        /**
         * 1.rocketMQ用的是ThreadPoolExecutor
         *
         * 2.ThreadPoolExecutor只有当任务队列满了才会启动大于核心线程数的新线程
         *
         * 3.rocketMQ传给ThreadPoolExecutor的队列是没设置容量的无界队列
         *
         * 4.所以任务队列基本不会满
         *
         * 5.所以ThreadPoolExecutor基本不会新建大于核心线程数的线程
         *
         * 6.所以rocketMQ设置的ConsumeThreadMax是无效的
         */
        consumer.setConsumeThreadMax(50);
        consumer.setConsumeThreadMin(50);

        /**
         * 设置广播模式加上这行代码
         */
//        consumer.setMessageModel(MessageModel.BROADCASTING);


        consumer.registerMessageListener(ledgerConsumerListener);
        log.info("ledger消费者启动成功");
        return consumer;
    }

}
